import subprocess
import time
import json
import typer

from intctl.status import StatusManager
from .utils import Spinner

def run(cmd: str, input: str = None) -> subprocess.CompletedProcess:
    return subprocess.run(
        cmd, shell=True, input=input, capture_output=True, text=True
    )

def setup_https_gateway(cfg: dict, status: StatusManager):
    """
    Sets up a GKE Gateway, a Google-managed certificate via nip.io,
    and an HTTPRoute to enable HTTPS for the gateway-manager.
    """
    status.start("https_gateway")

    project = cfg["project_id"]
    region = cfg["region"]
    workspace = cfg["workspace_uuid"]
    ip_name = f"gateway-manager-ip-{workspace}".lower()
    gateway_name = f"intellithing-gateway-{workspace}".lower()
    
    # 1. Ensure the GKE Gateway Controller is enabled
    print("🚀 Ensuring GKE Gateway API controller is enabled...")
    with Spinner("Checking GKE Gateway addons..."):
        run(
            "gcloud container clusters update int-" + workspace.lower() +
            f" --project={project} --region={region} "
            "--update-addons=Gateway=ENABLED"
        )
    print("✅ GKE Gateway API is enabled.")

    # 2. Get the Static IP Address
    print(f"🔍 Fetching static IP address for '{ip_name}'...")
    ip_result = run(f"gcloud compute addresses describe {ip_name} --region={region} --project={project} --format='value(address)'")
    if ip_result.returncode != 0:
        print(f"❌ Could not find static IP '{ip_name}'. Please run setup first.")
        status.fail("https_gateway", f"Static IP not found: {ip_name}")
        raise typer.Exit(1)
    
    static_ip = ip_result.stdout.strip()
    nip_domain = f"{static_ip.replace('.', '-')}.nip.io"
    print(f"✅ Found static IP: {static_ip}")
    print(f"✅ Using magic domain: {nip_domain}")

    # 3. Create the Gateway, ManagedCertificate, and HTTPRoute manifests
    manifests = f"""
apiVersion: gateway.networking.k8s.io/v1
kind: Gateway
metadata:
  name: {gateway_name}
  namespace: intellithing
  annotations:
    networking.gke.io/certmap: {gateway_name}-certmap # GKE will create a certmap with this name
spec:
  gatewayClassName: gke-l7-gxlb
  listeners:
  - name: https
    protocol: HTTPS
    port: 443
    allowedRoutes:
      namespaces:
        from: All
  addresses:
  - type: NamedAddress
    value: {ip_name}
---
apiVersion: networking.gke.io/v1
kind: ManagedCertificate
metadata:
  name: {gateway_name}-cert
  namespace: intellithing
spec:
  domains:
  - {nip_domain}
---
apiVersion: gateway.networking.k8s.io/v1
kind: HTTPRoute
metadata:
  name: gateway-manager-route
  namespace: intellithing
spec:
  parentRefs:
  - name: {gateway_name}
  hostnames:
  - "{nip_domain}"
  rules:
  - backendRefs:
    - name: gateway-manager
      port: 80
"""

    print("☸️ Applying Kubernetes Gateway manifests...")
    with Spinner("Applying Gateway, Certificate, and Route..."):
        apply_result = run("kubectl apply -f -", input=manifests)
    
    if apply_result.returncode != 0:
        print("❌ Failed to apply Gateway manifests.")
        print(apply_result.stderr)
        status.fail("https_gateway", "kubectl apply failed")
        raise typer.Exit(1)

    print("✅ Gateway manifests applied successfully.")
    print("⏳ Waiting for certificate to be provisioned. This can take up to 15 minutes...")

    # 4. Wait for the certificate to become active
    for i in range(45): # Poll for up to 15 minutes
        with Spinner(f"Checking certificate status... (Attempt {i+1}/45)"):
            cert_status_result = run(
                f"kubectl get managedcertificate {gateway_name}-cert -n intellithing -o json"
            )
        
        if cert_status_result.returncode == 0:
            cert_status = json.loads(cert_status_result.stdout)
            status_condition = next((c for c in cert_status.get("status", {}).get("conditions", []) if c.get("type") == "Ready"), None)
            if status_condition and status_condition.get("status") == "True":
                print(f"\n🎉 Certificate is ACTIVE! Your service is available at:")
                print(f"   https://{nip_domain}")
                status.complete("https_gateway")
                return
        
        time.sleep(20)

    print("\n❌ Certificate did not become active in time. Please check the status manually:")
    print(f"   kubectl describe managedcertificate {gateway_name}-cert -n intellithing")
    status.fail("https_gateway", "Certificate provisioning timed out")